module PlanetExpressExpress
  require 'date'
  class ShipmentReader
    def initialize(prices)
      @prices = prices
      @id = 0
    end

    # Expects to read a file
    def read_shipments(io)
      shipments = {
        S: [],
        M: [],
        L: [],
        all: []
      }

      current_month = 0
      current_year = 0

      io.each_line do |line|
        shipment = parse(line)

        if shipment[:date].year != current_year || shipment[:date].month != current_month
          close_month(shipments)
        end
        shipments[shipment[:size]].last.push(shipment)
        shipments[:all].last.push(shipment)

        current_year = shipment[:date].year
        current_month = shipment[:date].month
      end

      shipments
    end

    private

    attr_reader :prices

    def close_month(shipments)
      shipments.keys.each do |key|
        shipments[key].push([])
      end
    end

    def parse(line)
      parts = line.split(" ")

      shipment = {
        id: sequential_id,
        date: Date.strptime(parts[0], "%Y-%m-%d"),
        size: parts[1]&.to_sym,
        provider: parts[2]&.to_sym,
        price: 0,
        discount: 0,
        valid: true
      }

      shipment[:price] = prices[
        [
          shipment[:provider], shipment[:size]
        ]
      ]

      shipment
    end

    def sequential_id
      @id += 1
    end
  end
end
